#lesson.4.r
#function list: apply, summary, findInterval tapply, outer, sweep

#Note:  specific functions can be created and used in exactly 
#the same way as R-system functions 

#example 1 
xsq <- function(x) {x^2}
xsq(9.5)
xsq(1:10)

#example 2
#approx normal probabilities
P.norm <- function(z) {(1+sqrt(1-exp(-2*z^2/pi)))/2}
c(P.norm(1.645),pnorm(1.645))  
#approximate and exact probabilities
cbind(P.norm(-3:3),pnorm(-3:3))

#example 3 
#confidence interval (95%)
ci <- function(m,v) {
lower <- m-1.960*sqrt(v)
upper <- m+1.960*sqrt(v)
cbind(lower,m,upper)
}
m <- 10
v <- 2
ci(m,v)

# apply command to generate functions of values in an array
m <- matrix(c(21,12,24,12,4,5,11,1,18),3,3)
m

apply(m,1,sum) #rows
apply(m,2,sum) #columns
apply(m,1,mean) #rows
apply(m,2,mean) #columns
apply(m,1,sd) #rows
apply(m,2,sd) #columns

#example 4
#data
data <- matrix(rnorm(1000,2,4),10,100)
R1  <- apply(data,2,max)
R2  <- apply(data,2,min)
R <- R1-R2
mean(R)
median(R)
var(R)
summary(R)

#example 5
#accuracy of approximate variance of the estimated median
#version 1
n <- 50 #sample size
m <- 10 #mean value
s <- 2 #standard deviation
iter <- 2000
M <- matrix(rnorm(iter*n,m,s),n,iter)
dim(M)  #dimensions of M 
S <- apply(M,2,median)
median(S)
mean(S)
var(S)
(s^2*pi)/(2*n) #approximate variance of the median

#example 6  #distribution of three uniform values
#U = u1+u2+u3
nvalues <- runif(3*10000)
mvalues <- matrix(nvalues,3,10000)
mbar <- apply(mvalues,2,sum)
summary(mbar)

#example 7 
#do small values from a normal distribution 
#have a Poisson distribution?

p.count <- function(x) {
c <- mean(x)+3*sd(x)
sum(ifelse(x>c,1,0))
}
R <- matrix(rnorm(100*10000,2,2),100,10000)
dim(R)
count <- apply(R,2,p.count)
mean(count)
var(count)

#example 8 
#distribution of the harmonic mean
h.mean <- function(x) {length(x)/sum(1/x)} #harmonic mean value
n <- 12
iter <- 5000
data <- matrix(rnorm(n*iter,10,2),n,iter)
dim(data)
m <- apply(data,2,h.mean)
summary(m)
mean(m)
var(m)
hist(m,70)  #histogram -- graphic functions described in lesson 9

#findInterval -- classifies values into specified categories
#example:  random uniform distributed values
n <- 10000
bounds <- c(0.2,0.4,0.6,0.8)
dist <- findInterval(runif(n),bounds)
table(dist)
table(dist)/n

#classifies normally distributed value (mean=0 and variance=1)
n <- 2000
bounds <- seq(-2,2,0.5)
z <- rnorm(n)
dist <- findInterval(z,bounds)
table(dist)
table(dist)/n

#tapply -- calculation from values with the same index values 
n <- 200
z <- rnorm(n,10,2)
id <- findInterval(z,c(0,8:12,20))
table(id)
round(tapply(z,id,length),3)
round(tapply(z,id,sum),3)
round(tapply(z,id,mean),3)
round(tapply(z,id,sd),3)

#random normal distributed values
n <- 10000
z <- rnorm(n,10,2)
m <- mean(z)
s <- sd(z)
cbind(m,s)
cc <- m+c(-10*s,-2*s,-s,0,s,2*s,10*s)  #bounds
cc
z.tab <- findInterval(z,cc)
round(tapply(z,z.tab,length),3)
round(tapply(z,z.tab,mean),3)
round(tapply(z,z.tab,var),3)
tab <- table(z.tab)
tab
round(tab/sum(tab),3)
round(cumsum(tab)/sum(tab),3)

#sweep -- sweep(matrix,values,operator) uses input values 
#to calculate row or column values in an array

#data
m <- matrix(1:24,4,6)
m

sweep(m,2,2,"/")
sweep(m,1,2,"/")
sweep(m,2,2,"*")
sweep(m,1,3,"*")
sweep(m,1,c(1,2,3,4),"*")
sweep(m,1,c(1,2,3,4),"+")
fact <- apply(m,2,sum)
fact
sweep(m,2,fact,"/")
sweep(m,2,apply(m,2,sum),"/")


#order.r -- locates specific ranked values 
#data
y <- c(26,156,532,5,333,769,212,572,932,84)/100
y
r <- rank(y)
o <- order(y)
#rank 5 = position =?
o[5]
y[7]
y[o[5]]
o[1]
smallest <- y[o[1]]
o[length(y)]
largest <- y[o[length(y)]]
c(smallest,largest)
cbind(r,o,y)
#note
Y <- y[o]
YY <- sort(y)
cbind(Y,YY)

#solution to a polynomial equation
#for example -- x^2-8*x-20
x <- seq(0,5,0.00001)
fx <- function(x) {x^3-8*x-20}
f <- fx(x)
cbind(min(f),max(f))

plot(x,f,type="l",main="X^3-8*X^2-20") 
#more details of R-graphics lesson.6.r
o <- order(f)
x0 <- o[1]
c(x[x0],f[x0])
points(x[x0],f[x0],pch="x",cex=1.5)

#example 9
ht <- c(72,68,67,74,65)
wt <- c(160,180,170,100,240)
n <- length(ht)
cbind(ht,wt)
ht0 <- sort(ht)
ht0
o <- order(ht)
o
#maximum hight 
cbind(ht[o[n]], wt[o[n]])
cbind(ht[o],wt[o])


#outer.r -- applies functions to all possible pairs
x <- 1:3
y <- 1:4
outer(x,y,"*") #default is "*"
outer(x,y,"+")
outer(x,y,"-")
outer(x,y,"/")

#example 1
x <- 1:4
tab <- outer(x,x,"+")
tab
table(tab)

#example 2  gamma correlation coefficient 
y <- c(1,3,5,7,9)
x <- c(3,4,8,1,2)
xx <- outer(x,x,"-")
xx
yy <- outer(y,y,"-")
yy
C <- sum(sign(xx*yy>0))/2
D <- sum(sign(xx*yy<0))/2
G <- (C-D)/(C+D)
cbind(C,D,G)

#chi-square goodness-of-fit statistic
obs <- rbind(c(1,2,3,4,5),c(5,4,3,2,1))
obs
ex <- outer(apply(obs,1,sum),apply(obs,2,sum),"*")/sum(obs)
ex
X2 <- sum((obs-ex)^2/ex)
X2


#which -- locates specific values
d <- 1:20
d
which(d>10)
which(d<5 | d>15)
which(rep(c(0,1),10)==0)
x <- runif(10)
cbind(1:10,x)
m <- which(x==max(x))
cbind(m,x[m])
m <- which(x==min(x))
cbind(m,x[m])

#which numbers are even?
n<-c(35,22,23,28,25,3,40,8,15,2,27,9)
n
m <- which((n/2-trunc(n/2))==0)
m
n[m]

unk <- c(1,2,NaN,3,4,NaN,5,6)
which(unk=="NaN")
#version 1
unk[which(unk=="NaN")]<- -9
unk
#version 2
ifelse(unk=="NaN",-9,unk)


#which applies to an array
m <- matrix(1:12,3,4)
m
id <- which(m == 6,arr.ind=TRUE)
id
m[id]

locate <- which(m==max(m),arr.ind=TRUE)
locate
m[locate]
locate <- which(m==min(m),arr.ind=TRUE)
locate
m[locate]
locate <- which(m>8,arr.ind=TRUE)
locate
m[locate]


